import meager.router
import meager.http
import meager.logger
import json
import socketserver

class RequestHandler(socketserver.BaseRequestHandler):
    def handle(self):
        self.data = self.request.recv(1024).decode("utf-8")
        meager.logger.log(__class__, f"Got request from {self.client_address[0]}")
        parsed = meager.http.parse(self.data)
        route_match = self.server._router.match_request(parsed["url"])
        response = {
            "status": "OK 200",
            "http-version": "HTTP/1.1",
            "content-type": "text/html"
            }

        if(route_match):
            kwargs, function, server_options = route_match
            for key, value in server_options.items():
                response[key] = value
            response["content"] = function({"post": kwargs, "ip": self.client_address[0],"request": parsed})
            self.request.sendall(meager.http.build_response(response).encode("utf-8"))
        else:
            self.request.sendall(b"HTTP/1.1 OK 404\r\nContent-Type: text/html\r\n\r\n<h1>404 not found</h1>")

class Server(object):
    def __init__(self, port=2920, host="0.0.0.0"):
        self.port = port
        self.host = host
        self.router = meager.router.Router()

    def serve(self):
        with socketserver.TCPServer((self.host, self.port), RequestHandler) as sockserv:
            sockserv._router = self.router
            sockserv.serve_forever()



import json
def headers(header_string):
    tmp = {}
    return tmp

def parse(request):
    parsed = {}
    parsed["method"] = request.split()[0]
    parsed["url"] = request.split()[1]
    headers(request)
    return parsed

def build_response(response_dict):
    http_version = response_dict["http-version"]
    http_status = response_dict["status"]
    response_str = f"{http_version} {http_status}"

    if(isinstance(response_dict["content"], dict)):
        response_dict["content-type"] = "application/json"
        response_dict["content"] = json.dumps(response_dict["content"])

    for key, value in response_dict.items():
        if(key == "content"):
            response_str += "\r\n\r\n" + value
        else:
            response_str = response_str + "\r\n" + key + ": " + value
    return response_str


import re
class Router(object):
    def __init__(self):
        self.routes = []

    def create_route_expression(self, route):
        expression = re.sub(r'(<\w+>)', r'(?P\1.+)', route)
        return re.compile(f"^{expression}$")

    def register_route(self, url, func, server_options):
        pattern = self.create_route_expression(url)
        self.routes.append((pattern, func, server_options))

    def route(self, url, server_options={}, **options):
        def decorator(func):
            self.register_route(url, func, server_options)
            return func
        return decorator

    def match_request(self, url):
        for routexp, func, server_options in self.routes:
            m = routexp.match(url)
            if(m):
                return m.groupdict(), func, server_options
        return None


import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(name="meager",
    version="0.0.2",
    description="A really lightweight and simple API server, built on the socketserver module",
    url="https://thevoxel.net/projects/meager",
    author="Vorap",
    author_email="vorap@thevoxel.net",
    long_description_content_type="text/markdown",
    license="GNU GPLv3",
    packages=setuptools.find_packages(),
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
    ]
)